#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-08-02
#
# Monitor status of Check Point destktop policy server (dtps)
#
# 2020-06-07: code cleanup
# 2020-11-19: changed scanfunction to match other checks
# 2021-08-23: rewritten for CMK 2.0
#
# sample snmpwalk
# dtps activated
# .1.3.6.1.4.1.2620.1.9.1.0 = STRING: "Policy Server"
# .1.3.6.1.4.1.2620.1.9.2.0 = INTEGER: 6
# .1.3.6.1.4.1.2620.1.9.3.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.9.4.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.9.5.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.9.101.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.9.102.0 = STRING: "Up"
# .1.3.6.1.4.1.2620.1.9.103.0 = STRING: "Policy Server is up"
#
# dtps not activated
#
# .1.3.6.1.4.1.2620.1.9.1.0 = ""
# .1.3.6.1.4.1.2620.1.9.2.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.9.3.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.9.4.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.9.5.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.9.101.0 = INTEGER: 2
# .1.3.6.1.4.1.2620.1.9.102.0 = STRING: "Down"
# .1.3.6.1.4.1.2620.1.9.103.0 = STRING: "Policy Server is down"
#
# sample string_table
# dtps activated with trial license
# [[u'Policy Server', u'6', u'0', u'4294967294', u'0', u'0', u'OK', u'Policy Server is up']]
#
# dtps deactivated
# [[u'Policy Server', u'6', u'0', u'0', u'0', u'2', u'Down', u'Policy Server is down']]
#
# dtps never activated
# [[u'', u'0', u'0', u'0', u'0', u'2', u'Down', u'Policy Server is down']]
#

from dataclasses import dataclass
from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    check_levels,
    SNMPTree,
    startswith,
    all_of,
    any_of,
    matches,
)


@dataclass
class CheckpintDtps:
    dtpsprodname: str
    dtpslicensedusers: int
    dtpsconnectedusers: int
    dtpsstatcode: int
    dtpsversion: str
    dtpsstatshortdescr: str
    dtpsstatlongdescr: str


def parse_checkpoint_dtps(string_table: StringTable) -> Optional[CheckpintDtps]:
    try:
        dtpsprodname, dtpsvermajor, dtpsverminor, dtpslicensedusers, dtpsconnectedusers, dtpsstatcode, \
        dtpsstatshortdescr, dtpsstatlongdescr = string_table[0]
    except (IndexError, ValueError):
        return

    return CheckpintDtps(
        dtpsprodname=dtpsprodname,
        dtpsversion=f'{dtpsvermajor}.{dtpsverminor}',
        dtpslicensedusers=int(dtpslicensedusers),
        dtpsconnectedusers=int(dtpsconnectedusers),
        dtpsstatcode=int(dtpsstatcode),
        dtpsstatshortdescr=dtpsstatshortdescr,
        dtpsstatlongdescr=dtpsstatlongdescr,
    )


def discovery_checkpoint_dtps(section: CheckpintDtps) -> DiscoveryResult:
    yield Service()


def check_checkpoint_dtps(params, section: CheckpintDtps) -> CheckResult:
    yield Result(state=State.OK, summary=f'Status: {section.dtpsstatshortdescr}')
    yield Result(state=State.OK, summary=f'Version: {section.dtpsversion}')
    yield Result(state=State.OK, summary=f'Licensed users: {section.dtpslicensedusers}')

    if section.dtpslicensedusers == 4294967294:
        yield Result(state=State.OK, summary='(probably trial version)')

    yield from check_levels(
        value=section.dtpsconnectedusers,
        label='Connected users',
        levels_upper=params.get('dtpsusers', [None, None]),
        metric_name='dtpsconnectedusers',
        render_func=lambda v: f'{v:d}'
    )


register.snmp_section(
    name='checkpoint_dtps',
    parse_function=parse_checkpoint_dtps,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.9',  # CHECKPOINT-MIB::dtps
        oids=[
            '1',  # dtpsProdName
            '2',  # dtpsVerMajor
            '3',  # dtpsVerMinor
            '4',  # dtpsLicensedUsers
            '5',  # dtpsConnectedUsers
            '101',  # dtpsStatCode
            '102',  # dtpsStatShortDescr
            '103',  # dtpsStatLongDescr
        ]
    ),
    detect=all_of(
        any_of(
            startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
            all_of(
                equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
                equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
            )
        ),
        matches('.1.3.6.1.4.1.2620.1.9.1.0', 'Policy Server'),
    )
)

register.check_plugin(
    name='checkpoint_dtps',
    service_name='Desktop policy server',
    discovery_function=discovery_checkpoint_dtps,
    check_function=check_checkpoint_dtps,
    check_default_parameters={
        'dtpsusers': (10, 20)
    },
    check_ruleset_name='checkpoint_dtps',
)
